#!/bin/bash --posix
#
# Test of checkpoint/restart of a bash script.
# The two inner-most shells here are the actual tested code.
# The first nested shell checkpoints and restarts the
# inner-most and then finally checkpoints itself.
# Outer-most shell is just to discard the stderr generated
# when its child dies from a self-inflicted SIGTERM.
#
# In addition to checkpointing and restarting bash, this
# test also tests checkpoint/restart of the cr_restart
# and cr_checkpoint executables.  The cr_restart command is
# tested both as "root" of a checkpoint (in Context2)
# and as child of the shell (in Context3).  The cr_checkpoint
# command (for Context3) checkpoints itself.
#
# As an added wrinkle, we remove the file Context1 as a way
# to verify that the first cr_restart is not included in
# Context2 (the restart of Context2 would fail if the first
# cr_restart was present, since it would fail to reopen its
# input).  The open of Context1 is done with redirection, rather
# than a command line argument, to eliminate the race between
# the open() in cr_restart and the unlink() by rm.
#
top_builddir=${top_builddir:-..}
export top_builddir
rm -f Context[123] .Context[123].tmp
bash 2>/dev/null <<-'__EOF__'
	$top_builddir/util/cr_run/cr_run bash --posix -c '
		bash -c "for ((i=0; \$i<=15; ++i)); do printf \"%03d Hello\\n\" \$i; sleep 1; done" &
		pid=$!;
		sleep 3;
		# Checkpoint original child;
		$top_builddir/util/cr_checkpoint/cr_checkpoint --file=Context1 --term $pid;
		sleep 1;
		# Restart 1;
		$top_builddir/util/cr_restart/cr_restart /proc/self/fd/0 <Context1 &
		pid=$!;
		# Remove Context1;
		rm -f Context1;
		sleep 3;
		# Checkpoint restarted child;
		$top_builddir/util/cr_checkpoint/cr_checkpoint --file=Context2 --tree --term $pid;
		sleep 1;
		# Restart^2;
		$top_builddir/util/cr_restart/cr_restart Context2 &
		pid=$!;
		sleep 3;
		# Checkpoint self;
		$top_builddir/util/cr_checkpoint/cr_checkpoint --file=Context3 --term --tree $$;
		wait;
		echo "016 DONE"';
	# Restart shell
	$top_builddir/util/cr_restart/cr_restart Context3
	__EOF__
rm -f Context[123] .Context[123].tmp
